@@ -30,14 +30,6 @@
 
 function semilogy (varargin)
 
-  ## XXX FIXME XXX -- these plot states should really just be set
-  ## temporarily, probably inside an unwind_protect block, but there is
-  ## no way to determine their current values.
-
-#  gset nologscale x;
-#  gset logscale y;
-#  gset nopolar;
-
   __plt__ ("semilogy", varargin{:});
 
 endfunction
