@@ -1,5 +1,6 @@
-// Author: Paul Kienzle, 2001-03-22
+// Author: Paul Kienzle
 // I grant this code to the public domain.
+// 2001-03-22
 //
 // THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 // ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -397,7 +398,7 @@ public:
 #else
     // 2.1.53
     if (pmap) 
-      pmap->assign(octave_value(index), 
+      pmap->assign(octave_value(index+1), 
 		      key, Cell(value->as_octave_value()));
 #endif
     if (error_state) __mex->abort();
@@ -557,6 +558,7 @@ mxArray *mex::make_value(int nr, int nc, int cmplx)
   value->rows(nr);
   value->columns(nc);
   value->is_string(false);
+  value->map(NULL);
   value->name("");
 
   return value;
@@ -674,8 +676,9 @@ call_mex(callstyle cs, const octave_value_list& args, const int nargout)
   octave_value_list retval;
   if (! error_state)
     {
-      for (int i=0; i < nargout+1; i++)
+      for (int i=nargout; i >= 0; i--) {
 	if (argout[i]) retval(i) = argout[i]->as_octave_value();
+      }
       //retval(i) = argout[i] ? argout[i]->as_octave_value() : octave_value();
     }
 
@@ -967,7 +970,6 @@ extern "C" {
   mxArray *mxCreateStructMatrix(int nr, int nc, int num_keys, 
 				const char **keys)
   {
-    mexWarnMsgTxt("octave mex can't yet create structures\n");
     const string_vector ordered_keys(keys,num_keys);
     mxArray *m = __mex->make_value(nr, nc, ordered_keys);
     return m;
