@@ -17,6 +17,9 @@ without modification, or license it as you see fit.
 #include <octave/help.h>
 
 #include <tk.h>
+#ifndef CONST84
+#define CONST84
+#endif
 
 // I have attempted to block all access to octave variables from the tcl
 // thread while octave is running, but I didn't do it correctly and it
@@ -31,16 +34,21 @@ without modification, or license it as you see fit.
 #endif
 
 #if HAVE_BLT
+
 #include <blt.h>
 #include <bltVector.h>
-#endif /* HAVE_BLT */
+
+#endif /* !HAVE_BLT */
+
+#define _VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
+#define TCL_VERSION_NUMBER _VERSION(TCL_MAJOR_VERSION, TCL_MINOR_VERSION, TCL_RELEASE_SERIAL)
+#define TK_VERSION_NUMBER _VERSION(TK_MAJOR_VERSION, TK_MINOR_VERSION, TK_RELEASE_SERIAL)
 
 #ifndef HAVE_VTK
 #define HAVE_VTK 0
 #endif
 
 #if HAVE_VTK
-#include <vtk/vtk.h>
 #include <vtk/vtkRenderer.h>
 #include <vtk/vtkRenderWindow.h>
 #include <vtk/vtkRenderWindowInteractor.h>
@@ -48,7 +56,6 @@ without modification, or license it as you see fit.
 #include <vtk/vtkTransform.h>
 #include <vtk/vtkTransformPolyDataFilter.h>
 #include <vtk/vtkPoints.h>
-#include <vtk/vtkScalars.h>
 #include <vtk/vtkWarpScalar.h>
 #include <vtk/vtkDataSetMapper.h>
 #include <vtk/vtkPolyData.h>
@@ -57,24 +64,18 @@ without modification, or license it as you see fit.
 #endif /* HAVE_VTK */
 
 #include <string>
-#include <strstream>
-
-#include <string.h>
-#include <stdlib.h>
-#include <signal.h>
-#include <unistd.h>
+#include <iostream>
 #include <pthread.h>
-#include <iostream.h>
 
 #define TRUE  1
 #define FALSE 0
 
-#define ID "$Id: tk_interp.cc,v 1.1.1.1 2001/10/10 19:54:50 pkienzle Exp $"
+#define ID "$Id: tk_interp.cc,v 1.4 2004/02/06 19:33:19 pkienzle Exp $"
 
 static Tcl_Interp *interp = NULL;
 
 static char *command_to_do  = NULL;
-static char *command_result = NULL;
+static const char *command_result = NULL;
 
 static pthread_t       tk_thread = 0;
 static pthread_cond_t  tk_cond   = PTHREAD_COND_INITIALIZER;
@@ -140,7 +141,7 @@ public:
 
 
 ************************************************************/
-static octave_value get_octave_value(char *name)
+static octave_value get_octave_value(const char *name)
 {
   octave_value def;
 
@@ -225,7 +226,7 @@ unsigned char *make_custom_colormap(Matrix m)
 
 static int
 myStringMatchOctaveMatrix 
-(int argc, char **argv, int *widthP, int *heightP, Tcl_Interp *interp)
+(int argc, CONST84 char **argv, int *widthP, int *heightP, Tcl_Interp *interp)
 {
   if (argc < 1) return FALSE;
 
@@ -246,8 +247,8 @@ StringMatchOctaveMatrix
 (Tcl_Obj *str, Tcl_Obj *format, int *widthP, int *heightP, Tcl_Interp *interp)
 {
    int argc;
-   char **argv;
-   if (Tcl_SplitList(interp, (char *) str, &argc, &argv) != TCL_OK)
+   CONST84 char **argv;
+   if (Tcl_SplitList(interp, (char *)str, &argc, &argv) != TCL_OK)
      return FALSE;
 
    int ret = myStringMatchOctaveMatrix(argc, argv, widthP, heightP, interp);
@@ -257,7 +258,7 @@ StringMatchOctaveMatrix
 
 static int
 myStringReadOctaveMatrix
-(Tcl_Interp *interp, int argc, char **argv, Tk_PhotoHandle imageHandle,
+(Tcl_Interp *interp, int argc, CONST84 char **argv, Tk_PhotoHandle imageHandle,
 int destX, int destY, int width, int height, int srcX, int srcY)
 {
   // find matrix containing octave image
@@ -272,7 +273,7 @@ int destX, int destY, int width, int height, int srcX, int srcY)
 
   // interpret image format info
   int indexed = FALSE;
-  char *colormap = "global::__current_color_map__";
+  const char *colormap = "global::__current_color_map__";
   while(--argc)
     {
       argv++;
@@ -375,7 +376,12 @@ int destX, int destY, int width, int height, int srcX, int srcY)
 	  block.pixelPtr[pixel_index+2] = colormap_data[3*color_index+2];
 	}
     }
+#if TK_VERSION_NUMBER >= _VERSION(8,4,0)
+  Tk_PhotoPutBlock(imageHandle, &block, destX, destY, width, height, 
+	TK_PHOTO_COMPOSITE_SET);
+#else
   Tk_PhotoPutBlock(imageHandle, &block, destX, destY, width, height);
+#endif
   free((void *) block.pixelPtr);
   free((void *) colormap_data);
   
@@ -388,7 +394,7 @@ StringReadOctaveMatrix
 int destX, int destY, int width, int height, int srcX, int srcY)
 {
    int argc;
-   char **argv;
+   CONST84 char **argv;
 
    if (Tcl_SplitList(interp, (char *) str, &argc, &argv) != TCL_OK)
      return FALSE;
@@ -408,7 +414,7 @@ int destX, int destY, int width, int height, int srcX, int srcY)
 
 ************************************************************/
 
-int oct_string(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
+int oct_string(ClientData clientData, Tcl_Interp *interp, int argc, CONST84 char **argv)
 {
   if (argc < 2)
     {
@@ -430,7 +436,7 @@ stringName options\"", NULL);
          argv[1], "\" defined.", NULL);
       return TCL_ERROR;
    }
-   string s = def.string_value();
+   std::string s = def.string_value();
    Tcl_AppendResult(interp, s.c_str(), NULL);
    return TCL_OK;
 }
@@ -444,7 +450,7 @@ stringName options\"", NULL);
 
 ************************************************************/
 
-int oct_matrix(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
+int oct_matrix(ClientData clientData, Tcl_Interp *interp, int argc, CONST84 char **argv)
 {
    if(argc < 3) 
    {
@@ -541,7 +547,7 @@ matrixName element row column\"", NULL);
 
 ************************************************************/
 
-int oct_mtov(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
+int oct_mtov(ClientData clientData, Tcl_Interp *interp, int argc, CONST84 char **argv)
 {
    if(argc != 7)
    {
@@ -584,9 +590,9 @@ matrixName vectorName startX startY sizeX sizeY\"", NULL);
       return TCL_ERROR;
    }
    Blt_Vector *v;
-   if(Blt_VectorExists(interp, argv[2]))
+   if(Blt_VectorExists(interp, (char *)argv[2]))
    {
-      if(Blt_GetVector(interp, argv[2], &v) != TCL_OK)
+      if(Blt_GetVector(interp, (char *)argv[2], &v) != TCL_OK)
       {
          Tcl_AppendResult(interp, "Unable to get pointer to BLT vector \"",
             argv[2], "\".", NULL);
@@ -601,7 +607,7 @@ matrixName vectorName startX startY sizeX sizeY\"", NULL);
    }
    else
    {
-      if(Blt_CreateVector(interp, argv[2], (sizeX * sizeY), &v) != TCL_OK)
+      if(Blt_CreateVector(interp, (char *)argv[2], (sizeX * sizeY), &v) != TCL_OK)
       {
          Tcl_AppendResult(interp, "Unable to create BLT vector \"",
             argv[2], "\".", NULL);
@@ -630,7 +636,7 @@ matrixName vectorName startX startY sizeX sizeY\"", NULL);
 
 ************************************************************/
 
-int oct_mtovtk(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
+int oct_mtovtk(ClientData clientData, Tcl_Interp *interp, int argc, CONST84 char **argv)
 {
    if(argc != 3)
    {
@@ -688,8 +694,9 @@ int oct_mtovtk(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
    int numPts = input->GetNumberOfPoints();
    vtkPoints *newPts = vtkPoints::New();
    newPts->SetNumberOfPoints(numPts);
-   vtkScalars *colors = vtkScalars::New();
-   colors->SetNumberOfScalars(numPts);
+   // XXX FIXME XXX color handling has changed
+   // vtkScalars *colors = vtkScalars::New();
+   // colors->SetNumberOfScalars(numPts);
 
    // Convert values from Octave matrix and store in VTK object
    float p[3];
@@ -700,19 +707,19 @@ int oct_mtovtk(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
       int col = int((p[1] + 0.5) * (float) (m.cols() - 1));
       p[2] = ((m.elem(row, col) - min) / (max - min)) - 0.5;
       newPts->SetPoint(k, p);
-      colors->SetScalar(k, p[2]);
+      // colors->SetScalar(k, p[2]);
    }   
 
    surface->CopyStructure(input);
    surface->SetPoints(newPts);
-   surface->GetPointData()->SetScalars(colors);
+   // surface->GetPointData()->SetScalars(colors);
 
    // Clean up VTK objects
    plane->Delete();
    transform->Delete();
    transF->Delete();
    newPts->Delete();
-   colors->Delete();
+   // colors->Delete();
 
    return TCL_OK;
 }
@@ -737,7 +744,7 @@ Tcl_Interp *get_tk_thread_interp(void)
 #endif
 
 static
-int oct_cmd(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
+int oct_cmd(ClientData clientData, Tcl_Interp *interp, int argc, CONST84 char **argv)
 {
    if(argc < 2)
    {
@@ -765,7 +772,7 @@ int oct_cmd(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
 }
 
 static
-int oct_quit(ClientData clientData, Tcl_Interp *interp, int argc, char **argv)
+int oct_quit(ClientData clientData, Tcl_Interp *interp, int argc, CONST84 char **argv)
 {
    if(argc != 1)
    {
@@ -821,7 +828,7 @@ static void tk_thread_process_start(void)
 
 
    Tk_Window mainw = Tk_MainWindow(interp);
-   char *name = Tk_SetAppName(mainw, "tk_octave");
+   const char *name = Tk_SetAppName(mainw, "tk_octave");
    char buf[40];
    sprintf(buf, "wm title . {%s}", name);
    Tcl_Eval(interp, buf);
@@ -1010,8 +1017,8 @@ See also: tk_start, tk_end, tk_cmd")
       pthread_mutex_unlock(&tk_mutex);
       while(command_to_do)
       {
-         cout << "Processing command: " << command_to_do << "\n";
-         const string octave_cmd = string(command_to_do);
+	 std::cout << "Processing command: " << command_to_do << "\n";
+         const std::string octave_cmd = std::string(command_to_do);
          int parse_status = 0;
 #if SAFE_VAR
 	 pthread_mutex_lock(&oct_mutex);
@@ -1020,7 +1027,7 @@ See also: tk_start, tk_end, tk_cmd")
 #if SAFE_VAR
 	 pthread_mutex_unlock(&oct_mutex);
 #endif
-         cout << "Finished\n";
+	 std::cout << "Finished\n";
 
          pthread_mutex_lock(&tk_mutex);
          tk_fifo.pop();
