@@ -1,12 +1,12 @@
-## Usage: R = nanfunc( which_func, a_matrix, default_for_func ) applies
-## 'which_func' to non-NaN values of 'a_matrix' columnwise, returning
-## 'default_for_func' if all values are NaN.
+## Usage: R = nanfunc( which_func, a_matrix, default_for_func ) 
+## applies 'which_func' to non-NaN values of 'a_matrix' columnwise, 
+## returning 'default_for_func' if all values are NaN.
 
-## Copyright (C) 2000 Daniel Calvelo Aros
+## Copyright (C) 2000 Daniel Calvelo Aros 
 ## 
 ## Do what you want with this code. No warranty whatsoever.
 ##
-## Author:  DCA (dcalvelo@phare.univ-lille2.fr)
+## Author:  Daniel Calvelo Aros <dcalvelo@phare.univ-lille2.fr>
 ## Description:  Generic function caller for dealing with NaNs as
 ##               missing data
 
