@@ -55,6 +55,21 @@ DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(octave_galois, "galois", "galois");
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(octave_galois, "galois");
 #endif
 
+
+#if defined (HAVE_OCTAVE_CONCAT) || defined (HAVE_OLD_OCTAVE_CONCAT)
+octave_value octave_galois::resize (const dim_vector& dv) const
+{ 
+  if (dv.length() > 2)
+    {
+      error ("Can not resize galois structure to NDArray");
+      return octave_value ();
+    }
+  galois retval (gval); 
+  retval.resize (dv(0), dv(1)); 
+  return new octave_galois (retval);
+}
+#endif
+
 octave_value_list
 octave_galois::dotref (const octave_value_list& idx)
 {
@@ -366,6 +381,23 @@ octave_galois::matrix_value (bool) const
   return retval;
 }
 
+#ifdef HAVE_ND_ARRAYS
+NDArray
+octave_galois::array_value (bool) const
+{
+  int nr = rows ();
+  int nc = columns ();
+  dim_vector dv(nr,nc);
+  NDArray retval (dv);
+
+  for (int i=0; i<nr; i++)
+    for (int j=0; j<nc; j++)
+      retval(i + j*nr) = gval(i,j);
+
+  return retval;
+}
+#endif
+
 void
 octave_galois::assign (const octave_value_list& idx,
 			       const galois& rhs)
@@ -511,6 +543,13 @@ octave_galois::save_binary (std::ostream& os, bool& save_as_floats)
   return true;
 }
 
+#ifdef HAVE_SWAP_BYTES
+static inline void swap_4_bytes (volatile void *ptr)
+{
+  swap_bytes <4> (ptr);
+}
+#endif
+
 bool 
 octave_galois::load_binary (std::istream& is, bool swap,
 				 oct_mach_info::float_format fmt)
