@@ -6,7 +6,7 @@
  * Do what you like with this code as long as you
  *     leave this copyright in place.
  *
- * $Id: conv2.cc,v 1.6 2003/12/12 22:05:38 pkienzle Exp $
+ * $Id: conv2.cc,v 1.7 2004/06/29 10:28:01 pkienzle Exp $
 
 ## 2000-05-17: Paul Kienzle
 ##    * change argument to vector conversion to work for 2.1 series octave
@@ -65,6 +65,8 @@ conv2 (MArray<T>& R, MArray<T>& C, MArray2<T>& A, Shape ishape)
       case SHAPE_VALID:
          outM= Am - Cm + 1;
          outN= An - Rn + 1;
+	 if (outM < 0) outM = 0;
+	 if (outN < 0) outN = 0;
          edgM= edgN= 0;
 	 break;
       }
@@ -165,6 +167,8 @@ conv2 (MArray2<T>&A, MArray2<T>&B, Shape ishape)
       case SHAPE_VALID:
          outM= Am - Bm + 1;
          outN= An - Bn + 1;
+	 if (outM < 0) outM = 0;
+	 if (outN < 0) outN = 0;
          edgM= edgN= 0;
 	 break;
       }
