@@ -1,7 +1,7 @@
 #define STATUS(x) do { if (debug) std::cout << x << std::endl << std::flush; } while (0)
 #define USE_DEFUN_INTERNAL 1
 // Temporary hack (I hope).  For some reason the interpreter
-// is not finding the send/send_error commands, so I install
+// is not finding the send/senderror commands, so I install
 // them by hand..
 
 #include <iomanip>
@@ -33,7 +33,7 @@ static char* context = NULL;
 
 static double timestamp = 0.0;
 inline void tic(void) { timestamp = octave_time(); }
-inline double toc(void) {return round(-1e6*(timestamp-double(octave_time())));}
+inline double toc(void) {return ceil(-1e6*(timestamp-double(octave_time())));}
 
 // XXX FIXME XXX --- surely this is part of the standard library?
 void
@@ -129,7 +129,6 @@ sigterm_handler(int /* sig */)
 static void
 daemonize(void)
 {
-  if (debug) return; /* Don't daemonize if debugging */
   if (fork()) exit(0);
   std::cout << "Octave pid: " << octave_syscalls::getpid() << std::endl;
   signal(SIGTERM,sigterm_handler);
@@ -329,7 +328,7 @@ process_commands(int channel)
 	if (debug) tic();
 #if 1
 	octave_value_list evalargs;
-	evalargs(1) = "send_error(lasterr);";
+	evalargs(1) = "senderror(lasterr);";
 	evalargs(0) = context;
 	feval("eval",evalargs,0);
 #else
@@ -383,18 +382,18 @@ process_commands(int channel)
 int channel = -1;
 
 #if USE_DEFUN_INTERNAL
-DEFUN_INTERNAL(send_error,args,,false,"\
+DEFUN_INTERNAL(senderror,args,,false,"\
 Send the given error message across the socket.  The error context\n\
 is taken to be the last command received from the socket.")
 #else
-DEFUN_DLD(send_error,args,,"\
+DEFUN_DLD(senderror,args,,"\
 Send the given error message across the socket.  The error context\n\
 is taken to be the last command received from the socket.")
 #endif
 {
   std::string str;
   const int nargin = args.length();
-  if (nargin != 1) str="send_error not called with error";
+  if (nargin != 1) str="senderror not called with error";
   else str = args(0).string_value();
 
   // provide a context for the error (but not too much!)
@@ -519,9 +518,16 @@ listen(...,debug|nodebug)\n\
    If debug, echo all commands sent across the connection.  If nodebug,\n\
    detach the process and don't echo anything.  You will need to use\n\
    kill directly to end the process. Nodebug is the default.\n\
+listen(...,fork|nofork)\n\
+   If fork, start new server for each connection.  If nofork, only allow\n\
+   one connection at a time. Fork is the default (depending on system).\n\
 ")
 {
+#if USE_DEFUN_INTERNAL
   install_builtin_function (Fsend, "send", "builtin send doc", false);
+  install_builtin_function (Fsenderror, 
+			    "senderror", "builtin senderror doc", false);
+#endif
 
 #if defined(__CYGWIN__)
   bool canfork = listencanfork();
@@ -541,8 +547,11 @@ listen(...,debug|nodebug)\n\
   if (error_state) return ret;
 
   debug = false;
-  if (nargin >= 2) {
-    std::string lastarg(args(nargin-1).string_value());
+  int options = 1;
+  
+  int hostarg = -1;
+  for (int k = 1; k < nargin; k++) {
+    std::string lastarg(args(k).string_value());
     if (error_state) return ret;
     lowercase(lastarg);
     if (lastarg == "debug") {
@@ -550,6 +559,12 @@ listen(...,debug|nodebug)\n\
       nargin--;
     } else if (lastarg == "nodebug") {
       nargin--;
+    } else if (lastarg == "fork") {
+      canfork = true;
+    } else if (lastarg == "nofork") {
+      canfork = false;
+    } else if (hostarg == -1) {
+      hostarg = k;
     } else {
       print_usage("listen");
     }
@@ -558,8 +573,8 @@ listen(...,debug|nodebug)\n\
   struct in_addr localhost;
   struct in_addr hostid;
   inet_aton("127.0.0.1",&localhost);
-  if (nargin >= 2) {
-    std::string host(args(1).string_value());
+  if (hostarg > 0) {
+    std::string host(args(hostarg).string_value());
     if (error_state) return ret;
     if (!inet_aton(host.c_str(),&hostid)) {
       error("listen: could not find host id for %s",host.c_str());
@@ -610,7 +625,7 @@ listen(...,debug|nodebug)\n\
 
   sigchld_setup();
 
-  daemonize();
+  if (!debug && canfork) daemonize();
       
   // XXX FIXME XXX want a 'sandbox' option which disables fopen, cd, pwd,
   // system, popen ...  Or maybe just an initial script to run for each
