@@ -24,7 +24,7 @@ this program with any Open Source program, as defined by the
 Open Source Initiative (www.opensource.org)
 
 
-$Id: complex_sparse_ops.cc,v 1.23 2003/12/22 15:13:23 pkienzle Exp $
+$Id: complex_sparse_ops.cc,v 1.35 2004/11/16 10:31:58 adb014 Exp $
 
 */
 
@@ -87,7 +87,8 @@ octave_complex_sparse::octave_complex_sparse (SuperMatrix A )
 
 octave_complex_sparse::octave_complex_sparse (void )
 {
-   DEBUGMSG("complex_sparse( void)");
+  DEBUGMSG("complex_sparse( void)");
+  X = create_SuperMatrix (0, 0, 0, (Complex *)0, 0, 0);
 }
 
 octave_complex_sparse::~octave_complex_sparse (void)
@@ -266,6 +267,15 @@ UNOPDECL (uminus, a )
    return v.uminus();
 }   
 
+#ifdef HAVE_OCTAVE_UPLUS
+UNOPDECL (uplus, a ) 
+{ 
+   DEBUGMSG("complex_sparse - uplus");
+   CAST_UNOP_ARG (const octave_complex_sparse&); 
+   return new octave_complex_sparse (v);;
+}   
+#endif
+
 SuperMatrix
 oct_complex_sparse_transpose ( SuperMatrix X ) {
    DEFINE_SP_POINTERS_CPLX( X )
@@ -510,6 +520,143 @@ octave_complex_sparse::subsref( const std::string SUBSREF_STRREF type,
    return retval;
 }
 
+#if defined (HAVE_OCTAVE_CONCAT) || defined (HAVE_OLD_OCTAVE_CONCAT)
+octave_value 
+octave_complex_sparse::resize (const dim_vector& dv) const
+{
+  DEBUGMSG("complex_sparse_resize");
+  if (dv.length() > 2) {
+    error ("Can not resize complex sparse matrix to NDArray");
+    return octave_value ();
+  }
+
+  SPARSE_RESIZE (Complex, Complex, complex_)
+
+  //dPrint_CompCol_Matrix("octave sparse", (SuperMatrix *) &X);
+  return new octave_complex_sparse (X);
+}
+#endif
+
+#ifdef CLASS_HAS_LOAD_SAVE
+bool 
+octave_complex_sparse::save_ascii (std::ostream& os, bool& infnan_warned, 
+			       bool strip_nan_and_inf)
+{
+  DEFINE_SP_POINTERS_CPLX( X )
+  int nnz = NCFX->nnz;
+
+  // use N-D way of writing matrix
+  os << "# nnz: "      << nnz << "\n";
+  os << "# rows: "     << Xnr << "\n";
+  os << "# columns: "  << Xnc << "\n";
+
+  // add one to the printed indices to go from
+  //  zero-based to one-based arrays
+   for (int j=0; j< Xnc; j++)  {
+      OCTAVE_QUIT;
+      for (int i= cidxX[j]; i< cidxX[j+1]; i++) {
+         os << ridxX[i]+1 << " "  << j+1 << " "
+	    << coefX[i]   << "\n";
+      }
+   }
+  
+  return true;
+}
+
+bool
+octave_complex_sparse::load_ascii (std::istream& is)
+{
+  bool success = true;
+  int nnz, cols, rows;
+  if ( extract_keyword (is, "nnz",     nnz)  &&
+       extract_keyword (is, "rows",    rows)  &&
+       extract_keyword (is, "columns", cols) ) {
+     ComplexMatrix tmp( nnz, 3);
+     is >> tmp;
+     ColumnVector ridxA= real( tmp.column(0) );
+     ColumnVector cidxA= real( tmp.column(1) );
+     ComplexColumnVector coefA= tmp.column(2);
+     X= assemble_sparse( cols, rows, coefA, ridxA, cidxA, 0);
+  }
+  else {
+     error("load: failed to load sparse value");
+     success= false;
+  }
+
+  return success;
+}
+
+bool 
+octave_complex_sparse::save_binary (std::ostream& os, bool& save_as_floats)
+{
+  DEFINE_SP_POINTERS_CPLX( X )
+
+  FOUR_BYTE_INT itmp;
+  // Use negative value for ndims to be consistent with other formats
+  itmp= -2;        os.write (X_CAST (char *, &itmp), 4);
+  itmp= Xnr;       os.write (X_CAST (char *, &itmp), 4);
+  itmp= Xnc;       os.write (X_CAST (char *, &itmp), 4);
+  itmp= NCFX->nnz; os.write (X_CAST (char *, &itmp), 4);
+
+  // add one to the printed indices to go from
+  //  zero-based to one-based arrays
+   for (int j=0; j< Xnc; j++)  {
+      OCTAVE_QUIT;
+      for (int i= cidxX[j]; i< cidxX[j+1]; i++) {
+         itmp= ridxX[i]+1; os.write (X_CAST (char *, &itmp), 4);
+         itmp= j+1;        os.write (X_CAST (char *, &itmp), 4);
+
+	 // TODO: how to manage save_as_floats?
+         os.write (X_CAST (char *, &coefX[i]), 16);
+      }
+   }
+  return true;
+}
+
+bool 
+octave_complex_sparse::load_binary (std::istream& is, bool swap,
+				 oct_mach_info::float_format fmt)
+{
+  FOUR_BYTE_INT nnz, cols, rows, tmp;
+  if (! is.read (X_CAST (char *, &tmp), 4))
+    return false;
+
+  if (tmp != -2) {
+    error("load: only 2D sparse matrices are supported");
+    return false;
+  }
+
+  if (! is.read (X_CAST (char *, &rows), 4))
+    return false;
+  if (! is.read (X_CAST (char *, &cols), 4))
+    return false;
+  if (! is.read (X_CAST (char *, &nnz), 4))
+    return false;
+
+  ColumnVector        ridxA( nnz ), cidxA( nnz );
+  ComplexColumnVector coefA( nnz );
+  for( int i=0; i<nnz; i++) {
+     FOUR_BYTE_INT cidx, ridx;
+     if (! is.read (X_CAST (char *, &ridx), 4))
+       return false;
+     ridxA(i)= ridx;
+
+     if (! is.read (X_CAST (char *, &cidx), 4))
+       return false;
+     cidxA(i)= cidx;
+
+     Complex coef;
+     if (! is.read (X_CAST (char *, &coef), 16))
+       return false;
+     coefA(i)= coef;
+  }
+
+  X= assemble_sparse( cols, rows, coefA, ridxA, cidxA, 0);
+
+  return true;
+}
+#endif
+
 octave_value
 octave_complex_sparse::do_index_op ( const octave_value_list& idx, int resize_ok) 
 {
@@ -613,6 +760,35 @@ octave_complex_sparse::print (std::ostream& os, bool pr_as_read_syntax ) const
 #endif                  
 } // print
 
+octave_value_list 
+octave_complex_sparse::find( void ) const
+{
+   DEBUGMSG("complex_sparse - find");
+   DEFINE_SP_POINTERS_CPLX( X )
+   int nnz = NCFX->nnz;
+
+   octave_value_list retval;
+   ColumnVector I(nnz), J(nnz);
+   ComplexColumnVector S(nnz);
+
+   for (int i=0,cx=0; i< Xnc; i++) {
+      OCTAVE_QUIT;
+      for (int j= cidxX[i]; j< cidxX[i+1]; j++ ) {
+         I( cx ) = (double) ridxX[j]+1;
+         J( cx ) = (double) i+1;
+         S( cx ) =          coefX[j];
+         cx++;
+      }
+   }
+
+   retval(0)= I;
+   retval(1)= J;
+   retval(2)= S;
+   retval(3)= (double) Xnr;
+   retval(4)= (double) Xnc;
+   return retval;
+}
+
 //
 // sparse by complex  operations
 //
@@ -1302,7 +1478,7 @@ SuperMatrix assemble_sparse( int n, int m,
                              int assemble_do_sum)
 {
    DEBUGMSG("complex_sparse - assemble_sparse");
-   ASSEMBLE_SPARSE( Complex )
+   ASSEMBLE_SPARSE( Complex, true )
 // oct_sparse_verify_supermatrix( X );
    return X;
 }      
@@ -1315,6 +1491,13 @@ SuperMatrix oct_matrix_to_sparse(const ComplexMatrix & A) {
    return X;
 }
 
+DEFCATOP_SPARSE_FN (sm_csm, complex_sparse, sparse, complex_sparse, 
+		    super_matrix, super_matrix, concat)
+DEFCATOP_SPARSE_FN (csm_sm, complex_sparse, complex_sparse, sparse, 
+		    super_matrix, super_matrix, concat)
+DEFCATOP_SPARSE_FN (csm_csm, complex_sparse, complex_sparse, complex_sparse, 
+		    super_matrix, super_matrix, concat)
+
 void install_complex_sparse_ops() {
    //
    // unitary operations
@@ -1322,7 +1505,9 @@ void install_complex_sparse_ops() {
    INSTALL_UNOP  (op_transpose, octave_complex_sparse, transpose);
    INSTALL_UNOP  (op_hermitian, octave_complex_sparse, hermitian);
    INSTALL_UNOP  (op_uminus,    octave_complex_sparse, uminus);
-
+#ifdef HAVE_OCTAVE_UPLUS
+   INSTALL_UNOP  (op_uplus,     octave_complex_sparse, uplus);
+#endif
    //
    // binary operations: sparse with scalar
    //
@@ -1401,6 +1586,10 @@ void install_complex_sparse_ops() {
    INSTALL_BINOP (op_mul,      octave_complex_sparse, octave_complex_sparse, cs_cs_mul);
    INSTALL_BINOP (op_mul,      octave_complex_sparse, octave_sparse,         cs_s_mul);
    INSTALL_BINOP (op_mul,      octave_sparse,         octave_complex_sparse, s_cs_mul);
+
+   INSTALL_SPARSE_CATOP (octave_sparse, octave_complex_sparse, sm_csm);
+   INSTALL_SPARSE_CATOP (octave_complex_sparse, octave_sparse, csm_sm);
+   INSTALL_SPARSE_CATOP (octave_complex_sparse, octave_complex_sparse, csm_csm);
 }
 
 // functions for splu and inverse
@@ -1576,6 +1765,42 @@ complex_sparse_inv_uppertriang( SuperMatrix U)
 
 /*
  * $Log: complex_sparse_ops.cc,v $
+ * Revision 1.35  2004/11/16 10:31:58  adb014
+ * HAVE_OCTAVE_UPLUS config option for backwards compatiability
+ *
+ * Revision 1.34  2004/11/15 10:26:55  adb014
+ * Add unary plus operators as no-op, due to recent change in octave
+ *
+ * Revision 1.33  2004/11/09 23:34:49  adb014
+ * Fix concatenation for recent octave core CVS changes
+ *
+ * Revision 1.32  2004/08/31 15:23:37  adb014
+ * Small build fix for the macro SPARSE_RESIZE
+ *
+ * Revision 1.31  2004/08/25 16:13:57  adb014
+ * Working, but inefficient, concatentaion code
+ *
+ * Revision 1.30  2004/08/03 14:45:30  aadler
+ * clean up ASSEMBLE_SPARSE macro
+ *
+ * Revision 1.29  2004/08/02 16:35:40  aadler
+ * saving to the octave -binary format
+ *
+ * Revision 1.28  2004/08/02 15:46:33  aadler
+ * tests for sparse saving
+ *
+ * Revision 1.27  2004/08/02 14:47:52  aadler
+ * complex save and load ascii
+ *
+ * Revision 1.26  2004/07/27 20:56:44  aadler
+ * first steps to concatenation working
+ *
+ * Revision 1.25  2004/07/27 18:24:10  aadler
+ * save_ascii
+ *
+ * Revision 1.24  2004/07/27 16:05:55  aadler
+ * simplify find
+ *
  * Revision 1.23  2003/12/22 15:13:23  pkienzle
  * Use error/return rather than SP_FATAL_ERROR where possible.
  *
