@@ -16,9 +16,30 @@ You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, write to the Free
 Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-$Id: make_sparse.h,v 1.22 2004/01/29 21:13:20 pkienzle Exp $
+$Id: make_sparse.h,v 1.29 2004/11/09 23:34:49 adb014 Exp $
 
 $Log: make_sparse.h,v $
+Revision 1.29  2004/11/09 23:34:49  adb014
+Fix concatenation for recent octave core CVS changes
+
+Revision 1.28  2004/08/25 16:13:57  adb014
+Working, but inefficient, concatentaion code
+
+Revision 1.27  2004/08/02 18:46:57  aadler
+some code for concat operators
+
+Revision 1.26  2004/08/02 15:46:33  aadler
+tests for sparse saving
+
+Revision 1.25  2004/07/27 20:56:44  aadler
+first steps to concatenation working
+
+Revision 1.24  2004/07/27 18:24:10  aadler
+save_ascii
+
+Revision 1.23  2004/07/27 16:05:55  aadler
+simplify find
+
 Revision 1.22  2004/01/29 21:13:20  pkienzle
 Use std::vector rather than std::auto_ptr for temporary memory
 
@@ -213,6 +234,12 @@ class ostream;
 
 #include <octave/utils.h>
 
+#ifdef CLASS_HAS_LOAD_SAVE
+#include <octave/byte-swap.h>
+#include <octave/ls-oct-ascii.h>
+#endif
+
+
 class Octave_map;
 class octave_value_list;
 
@@ -270,6 +297,29 @@ public:
    
    void print (std::ostream& os, bool pr_as_read_syntax = false) const ;
 
+   octave_value_list find( void ) const;
+
+#if defined (HAVE_OCTAVE_CONCAT) || defined (HAVE_OLD_OCTAVE_CONCAT)
+   octave_value resize (const dim_vector& dv) const;
+#endif
+
+#ifdef CLASS_HAS_LOAD_SAVE
+  bool save_ascii (std::ostream& os, bool& infnan_warned,
+		 bool strip_nan_and_inf);
+
+  bool load_ascii (std::istream& is);
+
+  bool save_binary (std::ostream& os, bool& save_as_floats);
+  
+  bool load_binary (std::istream& is, bool swap, 
+  		    oct_mach_info::float_format fmt);
+
+#if defined (HAVE_HDF5)
+// TODO
+#endif
+#endif
+
+
    type_conv_fcn numeric_conversion_function (void) const;
 
 private:
@@ -284,6 +334,7 @@ private:
 // sparse class definition
 //
 class
+//octave_sparse : public octave_base_value
 octave_sparse : public octave_base_value
 {
 public:
@@ -335,6 +386,28 @@ public:
 
    void print (std::ostream& os, bool pr_as_read_syntax = false) const ;
 
+   octave_value_list find( void ) const;
+
+#if defined (HAVE_OCTAVE_CONCAT) || defined (HAVE_OLD_OCTAVE_CONCAT)
+   octave_value resize (const dim_vector& dv) const;
+#endif
+
+#ifdef CLASS_HAS_LOAD_SAVE
+  bool save_ascii (std::ostream& os, bool& infnan_warned,
+		 bool strip_nan_and_inf);
+
+  bool load_ascii (std::istream& is);
+
+  bool save_binary (std::ostream& os, bool& save_as_floats);
+  
+  bool load_binary (std::istream& is, bool swap, 
+  		    oct_mach_info::float_format fmt);
+
+#if defined (HAVE_HDF5)
+// TODO
+#endif
+#endif
+
    type_conv_fcn numeric_conversion_function (void) const;
 
 private:
@@ -486,6 +559,12 @@ create_SuperMatrix( int nr, int nc, int nnz,
                     int * ridx,
                     int * cidx );
 
+#if defined (HAVE_OCTAVE_CONCAT) || defined (HAVE_OLD_OCTAVE_CONCAT)
+SuperMatrix 
+concat (const SuperMatrix& ra, const SuperMatrix& rb,
+	const Array<int>& ra_idx);
+#endif
+
 void
 LUextract(SuperMatrix *L, SuperMatrix *U, double *Lval, int *Lrow,
           int *Lcol, double *Uval, int *Urow, int *Ucol, int *snnzL,
