@@ -23,7 +23,7 @@ In addition to the terms of the GPL, you are permitted to link
 this program with any Open Source program, as defined by the
 Open Source Initiative (www.opensource.org)
 
-$Id: sparse_ops.cc,v 1.17 2003/12/22 15:13:23 pkienzle Exp $
+$Id: sparse_ops.cc,v 1.29 2004/11/16 10:31:58 adb014 Exp $
 
 */
 
@@ -49,6 +49,7 @@ create_SuperMatrix( int nr, int nc, int nnz,
 // assemble a sparse matrix from elements
 //   called by > 1 args for sparse
 // NOTE: index vectors are 1 based!
+//  m= rows, n=cols
 SuperMatrix assemble_sparse( int n, int m,
                              ColumnVector& coefA,
                              ColumnVector& ridxA,
@@ -56,7 +57,7 @@ SuperMatrix assemble_sparse( int n, int m,
                              int assemble_do_sum)
 {
    DEBUGMSG("sparse - assemble_sparse");
-   ASSEMBLE_SPARSE( double )
+   ASSEMBLE_SPARSE( double, true )
 // oct_sparse_verify_supermatrix( X );
    return X;
 }      
@@ -68,13 +69,14 @@ SuperMatrix assemble_sparse( int n, int m,
 
 octave_sparse::octave_sparse (void )
 {
-      DEBUGMSG("sparse( void)");
+  DEBUGMSG("sparse( void)");
+  X = create_SuperMatrix (0, 0, 0, (double *)0, 0, 0);
 }
 
 octave_sparse::octave_sparse (SuperMatrix A )
 {
-      DEBUGMSG("sparse( SuperMatrix A)");
-      X= A;
+  DEBUGMSG("sparse( SuperMatrix A)");
+  X= A;
 }
 
 octave_sparse::~octave_sparse (void)
@@ -227,6 +229,16 @@ UNOPDECL (uminus, a )
    return v.uminus();
 }   
 
+
+#ifdef HAVE_OCTAVE_UPLUS
+UNOPDECL (uplus, a ) 
+{ 
+   DEBUGMSG("sparse - uplus");
+   CAST_UNOP_ARG (const octave_sparse&); 
+   return new octave_sparse (v);
+}   
+#endif
+
 SuperMatrix
 oct_sparse_transpose ( SuperMatrix X ) {
    DEFINE_SP_POINTERS_REAL( X )
@@ -458,6 +470,261 @@ octave_sparse::subsref( const std::string SUBSREF_STRREF type,
    return retval;
 }
 
+#if defined (HAVE_OCTAVE_CONCAT) || defined (HAVE_OLD_OCTAVE_CONCAT)
+octave_value 
+octave_sparse::resize (const dim_vector& dv) const
+{
+  DEBUGMSG("sparse_resize");
+  if (dv.length() > 2) {
+    error ("Can not resize sparse matrix to NDArray");
+    return octave_value ();
+  }
+
+  SPARSE_RESIZE (double, , )
+
+  //dPrint_CompCol_Matrix("octave sparse", (SuperMatrix *) &X);
+  return new octave_sparse (X);
+}
+
+#define SPARSE_CONCAT \
+  for (int i = 0; i < An; i++) \
+    { \
+      if ((i < ra_idx(1)) || (Bn + ra_idx(1) <= i) || (Bnnz == 0)) \
+	{ \
+	  if (Annz != 0) \
+	    { \
+	      for (int j = Acolptr[i]; j < Acolptr[i+1]; j++, Cidx++) \
+		{ \
+		  Cnzval[Cidx] = Anzval[j]; \
+		  Crowind[Cidx] = Arowind[j]; \
+		} \
+	    } \
+	} \
+      else \
+	{ \
+	  if (Annz == 0) \
+	    { \
+	      for (int j = Bcolptr[i]; j < Bcolptr[i+1]; j++, Cidx++) \
+		{ \
+		  Cnzval[Cidx] = Bnzval[j]; \
+		  Crowind[Cidx] = Browind[j] + ra_idx(0); \
+		} \
+	    }\
+	  else \
+	    { \
+	      for (int j = Acolptr[i], k = Bcolptr[i-ra_idx(1)]; \
+		   (j < Acolptr[i+1]) || (k < Bcolptr[i+1-ra_idx(1)]); \
+		   Cidx++) \
+		{ \
+		  if (j < Acolptr[i+1] && k < Bcolptr[i+1-ra_idx(1)]) \
+		    if (Arowind [j] < Browind [k] + ra_idx(0)) \
+		      { \
+			Cnzval[Cidx] = Anzval[j]; \
+			Crowind[Cidx] = Arowind[j++]; \
+		      } \
+		    else \
+		      { \
+			if (Arowind [j] == Browind [k] + ra_idx(0)) \
+			  j++; \
+			Cnzval[Cidx] = Bnzval[k]; \
+			Crowind[Cidx] = Browind[k++] + ra_idx(0); \
+		      } \
+		  else if (j < Acolptr[i+1]) \
+		    { \
+		      Cnzval[Cidx] = Anzval[j]; \
+		      Crowind[Cidx] = Arowind[j++]; \
+		    } \
+		  else \
+		    { \
+		      Cnzval[Cidx] = Bnzval[k]; \
+		      Crowind[Cidx] = Browind[k++] + ra_idx(0); \
+		    } \
+		} \
+	    } \
+	} \
+      Ccolptr[i+1] = Cidx; \
+    } \
+  maybe_shrink (Cidx, Annz + Bnnz, Crowind, Cnzval); \
+  return create_SuperMatrix(ra.nrow, An, Cidx, Cnzval, Crowind, Ccolptr);
+
+SuperMatrix concat (const SuperMatrix& ra,
+		    const SuperMatrix& rb,
+		    const Array<int>& ra_idx)
+{
+  DEBUGMSG("sparse - concat");
+
+  NCformat * Astore  = (NCformat * ) ra.Store;
+  int Annz = Astore->nnz;
+  int * Arowind = Astore->rowind;
+  int * Acolptr = Astore->colptr;
+  int An = ra.ncol;
+
+  NCformat * Bstore  = (NCformat * ) rb.Store;
+  int Bnnz = Bstore->nnz;
+  int * Browind = Bstore->rowind;
+  int * Bcolptr = Bstore->colptr;
+  int Bn = rb.ncol;
+
+  int * Crowind = (int *) oct_sparse_malloc ((Annz + Bnnz) * sizeof(int));
+  int * Ccolptr = (int *) oct_sparse_malloc ((An + 1) * sizeof(int));
+  Ccolptr [0] = 0; 
+  int Cidx = 0;
+  
+  if ((ra.Dtype == _Z) || (rb.Dtype == _Z))
+    {
+      Complex * Cnzval = (Complex *) oct_sparse_malloc ((Annz + Bnnz) *
+							sizeof(Complex));
+      if ((ra.Dtype == _Z) && (rb.Dtype == _Z))
+	{
+	  Complex * Anzval = (Complex *)Astore->nzval;
+	  Complex * Bnzval = (Complex *)Bstore->nzval;
+	  SPARSE_CONCAT;
+	}
+      else if (ra.Dtype == _Z)
+	{
+	  Complex * Anzval = (Complex *)Astore->nzval;
+	  double * Bnzval = (double *)Bstore->nzval;
+	  SPARSE_CONCAT;
+	}
+      else if (rb.Dtype == _Z)
+	{
+	  double * Anzval = (double *)Astore->nzval;
+	  Complex * Bnzval = (Complex *)Bstore->nzval;
+	  SPARSE_CONCAT;
+	}
+    }
+  else
+    {
+      double * Cnzval = (double *) oct_sparse_malloc ((Annz + Bnnz) *
+						      sizeof(double));
+      double * Anzval = (double *)Astore->nzval;
+      double * Bnzval = (double *)Bstore->nzval;
+      SPARSE_CONCAT;
+    }
+}
+#endif
+
+#ifdef CLASS_HAS_LOAD_SAVE
+bool 
+octave_sparse::save_ascii (std::ostream& os, bool& infnan_warned, 
+			       bool strip_nan_and_inf)
+{
+  DEFINE_SP_POINTERS_REAL( X )
+  int nnz = NCFX->nnz;
+
+  // TODO: how should we manage infnan warnings?
+  // use 2-D way of writing matrix
+  os << "# nnz: "      << nnz << "\n";
+  os << "# rows: "     << Xnr << "\n";
+  os << "# columns: "  << Xnc << "\n";
+
+  // add one to the printed indices to go from
+  //  zero-based to one-based arrays
+   for (int j=0; j< Xnc; j++)  {
+      OCTAVE_QUIT;
+      for (int i= cidxX[j]; i< cidxX[j+1]; i++) {
+         os << ridxX[i]+1 << " "  << j+1 << " "
+	    << coefX[i]   << "\n";
+      }
+   }
+  
+  return true;
+}
+
+bool
+octave_sparse::load_ascii (std::istream& is)
+{
+  bool success = true;
+  int nnz, cols, rows;
+  if ( extract_keyword (is, "nnz",     nnz)  &&
+       extract_keyword (is, "rows",    rows)  &&
+       extract_keyword (is, "columns", cols) ) {
+     Matrix tmp( nnz, 3);
+     is >> tmp;
+     ColumnVector ridxA= tmp.column(0);
+     ColumnVector cidxA= tmp.column(1);
+     ColumnVector coefA= tmp.column(2);
+     X= assemble_sparse( cols, rows, coefA, ridxA, cidxA, 0);
+  }
+  else {
+     error("load: failed to load sparse value");
+     success= false;
+  }
+
+  return success;
+}
+
+bool 
+octave_sparse::save_binary (std::ostream& os, bool& save_as_floats)
+{
+  DEFINE_SP_POINTERS_REAL( X )
+
+  FOUR_BYTE_INT itmp;
+  // Use negative value for ndims to be consistent with other formats
+  itmp= -2;        os.write (X_CAST (char *, &itmp), 4);
+  itmp= Xnr;       os.write (X_CAST (char *, &itmp), 4);
+  itmp= Xnc;       os.write (X_CAST (char *, &itmp), 4);
+  itmp= NCFX->nnz; os.write (X_CAST (char *, &itmp), 4);
+
+  // add one to the printed indices to go from
+  //  zero-based to one-based arrays
+   for (int j=0; j< Xnc; j++)  {
+      OCTAVE_QUIT;
+      for (int i= cidxX[j]; i< cidxX[j+1]; i++) {
+         itmp= ridxX[i]+1; os.write (X_CAST (char *, &itmp), 4);
+         itmp= j+1;        os.write (X_CAST (char *, &itmp), 4);
+
+	 // TODO: how to manage save_as_floats?
+         os.write (X_CAST (char *, &coefX[i]), 8);
+      }
+   }
+  return true;
+}
+
+bool 
+octave_sparse::load_binary (std::istream& is, bool swap,
+				 oct_mach_info::float_format fmt)
+{
+  FOUR_BYTE_INT nnz, cols, rows, tmp;
+  if (! is.read (X_CAST (char *, &tmp), 4))
+    return false;
+
+  if (tmp != -2) {
+    error("load: only 2D sparse matrices are supported");
+    return false;
+  }
+
+  if (! is.read (X_CAST (char *, &rows), 4))
+    return false;
+  if (! is.read (X_CAST (char *, &cols), 4))
+    return false;
+  if (! is.read (X_CAST (char *, &nnz), 4))
+    return false;
+
+  ColumnVector ridxA( nnz ), cidxA( nnz ), coefA( nnz );
+  for( int i=0; i<nnz; i++) {
+     FOUR_BYTE_INT cidx, ridx;
+     if (! is.read (X_CAST (char *, &ridx), 4))
+       return false;
+     ridxA(i)= ridx;
+
+     if (! is.read (X_CAST (char *, &cidx), 4))
+       return false;
+     cidxA(i)= cidx;
+
+     double coef;
+     if (! is.read (X_CAST (char *, &coef), 8))
+       return false;
+     coefA(i)= coef;
+  }
+
+  X= assemble_sparse( cols, rows, coefA, ridxA, cidxA, 0);
+
+  return true;
+}
+#endif
+
+
 octave_value
 octave_sparse::do_index_op ( const octave_value_list& idx, int) 
 {
@@ -546,17 +813,47 @@ octave_sparse::print (std::ostream& os, bool pr_as_read_syntax ) const
                                  ", nnz=" << nnz << ")\n";
    // add one to the printed indices to go from
    //  zero-based to one-based arrays
-   for (int j=0; j< Xnc; j++) 
+   for (int j=0; j< Xnc; j++)  {
+      OCTAVE_QUIT;
       for (int i= cidxX[j]; i< cidxX[j+1]; i++) {
-	 OCTAVE_QUIT;
          os << "  (" << ridxX[i]+1 <<
                " , "  << j+1 << ") -> ";
 	 octave_print_internal( os, coefX[i], false );
 	 os << "\n";
       }
-#endif                  
+   }
+#endif
 } // print
 
+octave_value_list 
+octave_sparse::find( void ) const
+{
+   DEBUGMSG("sparse - find");
+   DEFINE_SP_POINTERS_REAL( X )
+   int nnz = NCFX->nnz;
+
+   octave_value_list retval;
+   ColumnVector I(nnz), J(nnz);
+   ColumnVector S(nnz);
+
+   for (int i=0,cx=0; i< Xnc; i++) {
+      OCTAVE_QUIT;
+      for (int j= cidxX[i]; j< cidxX[i+1]; j++ ) {
+         I( cx ) = (double) ridxX[j]+1;
+         J( cx ) = (double) i+1;
+         S( cx ) =          coefX[j];
+         cx++;
+      }
+   }
+
+   retval(0)= I;
+   retval(1)= J;
+   retval(2)= S;
+   retval(3)= (double) Xnr;
+   retval(4)= (double) Xnc;
+   return retval;
+}
+
 //
 // sparse by scalar  operations
 //
@@ -1066,6 +1363,9 @@ SuperMatrix oct_matrix_to_sparse(const Matrix & A) {
    return X;
 }
 
+DEFCATOP_SPARSE_FN (sm_sm, sparse, sparse, sparse, super_matrix, super_matrix,
+		    concat)
+
 void install_sparse_ops() {
    //
    // unitary operations
@@ -1073,6 +1373,9 @@ void install_sparse_ops() {
    INSTALL_UNOP  (op_transpose, octave_sparse, transpose);
    INSTALL_UNOP  (op_hermitian, octave_sparse, hermitian);
    INSTALL_UNOP  (op_uminus,    octave_sparse, uminus);
+#ifdef HAVE_OCTAVE_UPLUS
+   INSTALL_UNOP  (op_uplus,     octave_sparse, uplus);
+#endif
 
    //
    // binary operations: sparse with scalar
@@ -1105,6 +1408,8 @@ void install_sparse_ops() {
    INSTALL_BINOP (op_mul,      octave_sparse, octave_matrix, s_f_mul);
    INSTALL_BINOP (op_mul,      octave_matrix, octave_sparse, f_s_mul);
    INSTALL_BINOP (op_mul,      octave_sparse, octave_sparse, s_s_mul);
+
+   INSTALL_SPARSE_CATOP (octave_sparse, octave_sparse, sm_sm);
 }
 
 // functions for splu and inverse
@@ -1282,6 +1587,42 @@ sparse_inv_uppertriang( SuperMatrix U) {
 
 /*
  * $Log: sparse_ops.cc,v $
+ * Revision 1.29  2004/11/16 10:31:58  adb014
+ * HAVE_OCTAVE_UPLUS config option for backwards compatiability
+ *
+ * Revision 1.28  2004/11/15 10:26:55  adb014
+ * Add unary plus operators as no-op, due to recent change in octave
+ *
+ * Revision 1.27  2004/11/09 23:34:49  adb014
+ * Fix concatenation for recent octave core CVS changes
+ *
+ * Revision 1.26  2004/08/31 15:23:45  adb014
+ * Small build fix for the macro SPARSE_RESIZE
+ *
+ * Revision 1.25  2004/08/25 16:13:57  adb014
+ * Working, but inefficient, concatentaion code
+ *
+ * Revision 1.24  2004/08/03 14:45:31  aadler
+ * clean up ASSEMBLE_SPARSE macro
+ *
+ * Revision 1.23  2004/08/02 18:46:57  aadler
+ * some code for concat operators
+ *
+ * Revision 1.22  2004/08/02 16:35:41  aadler
+ * saving to the octave -binary format
+ *
+ * Revision 1.21  2004/08/02 15:46:33  aadler
+ * tests for sparse saving
+ *
+ * Revision 1.20  2004/07/27 20:56:44  aadler
+ * first steps to concatenation working
+ *
+ * Revision 1.19  2004/07/27 18:24:10  aadler
+ * save_ascii
+ *
+ * Revision 1.18  2004/07/27 16:05:55  aadler
+ * simplify find
+ *
  * Revision 1.17  2003/12/22 15:13:23  pkienzle
  * Use error/return rather than SP_FATAL_ERROR where possible.
  *
