@@ -16,9 +16,21 @@ You should have received a copy of the GNU General Public License
 along with Octave; see the file COPYING.  If not, write to the Free
 Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-$Id: sparse_ops.h,v 1.11 2003/12/22 15:13:23 pkienzle Exp $
+$Id: sparse_ops.h,v 1.15 2004/11/09 23:34:49 adb014 Exp $
 
 $Log: sparse_ops.h,v $
+Revision 1.15  2004/11/09 23:34:49  adb014
+Fix concatenation for recent octave core CVS changes
+
+Revision 1.14  2004/08/31 15:23:45  adb014
+Small build fix for the macro SPARSE_RESIZE
+
+Revision 1.13  2004/08/25 16:13:57  adb014
+Working, but inefficient, concatentaion code
+
+Revision 1.12  2004/08/03 14:45:31  aadler
+clean up ASSEMBLE_SPARSE macro
+
 Revision 1.11  2003/12/22 15:13:23  pkienzle
 Use error/return rather than SP_FATAL_ERROR where possible.
 
@@ -271,7 +283,7 @@ Revision 1.1  2001/03/30 04:34:23  aadler
 // The OCTAVE_LOCAL_BUFFER can't be used for the sort index
 //  when the requested size is too big. we need to put it on the
 //  heap.
-#define ASSEMBLE_SPARSE( TYPX ) \
+#define ASSEMBLE_SPARSE( TYPX , REJECT_OUT_OF_RANGE )  \
    int  nnz= MAX( ridxA.length(), cidxA.length() ); \
    TYPX* coefX= (TYPX*)oct_sparse_malloc( nnz  * sizeof(TYPX)); \
    int * ridxX= (int *)oct_sparse_malloc( nnz  * sizeof(int) ); \
@@ -287,15 +299,20 @@ Revision 1.1  2001/03/30 04:34:23  aadler
    for (int i=0; i<nnz; i++) { \
       double rowidx=  ( ri_scalar ? ridxA(0) : ridxA(i) ) - 1; \
       double colidx=  ( ci_scalar ? cidxA(0) : cidxA(i) ) - 1;  \
-      if (rowidx >= m || rowidx < 0) \
-         error("sparse row index out of range"); \
-      else if (colidx >= n || colidx < 0) \
-         error("sparse column index out of range"); \
-      if (error_state) { actual_nnz=0; break; } \
-      if ( coefA(cf_scalar?0:i) != 0. ) { \
-         sidx[actual_nnz].val = (long) ( rowidx + m*colidx ); \
-         sidx[actual_nnz].idx = i; \
-         actual_nnz++; \
+      if (rowidx < m && rowidx >= 0 && \
+          colidx < n && colidx >= 0 ) { \
+         if ( coefA( cf_scalar ? 0 : i ) != 0. ) { \
+            sidx[actual_nnz].val = (long) ( rowidx + m * colidx ); \
+            sidx[actual_nnz].idx = i; \
+            actual_nnz++; \
+         } \
+      } \
+      else { \
+	if( REJECT_OUT_OF_RANGE ) { \
+	  error("sparse (row,col) index (%d,%d) out of range", rowidx,colidx);\
+          actual_nnz=0; \
+	  break; \
+	} \
       } \
    } \
  \
@@ -312,16 +329,16 @@ Revision 1.1  2001/03/30 04:34:23  aadler
       long  val= (long) sidx[i].val; \
       if (prev_val < val) { \
          ii++; \
-         coefX[ii]=     ( cf_scalar ? coefA(0) : coefA(idx) ); \
-         double ri  =   ( ri_scalar ? ridxA(0) : ridxA(idx) ) - 1 ; \
+         coefX[ii]=     coefA( cf_scalar ? 0 : idx ); \
+         double ri  =   ridxA( ri_scalar ? 0 : idx ) - 1 ; \
          ridxX[ii]= (long) (ri - ((long) (ri/m))*m ) ; \
-         long ci  = (long)( ci_scalar ? cidxA(0) : cidxA(idx) ) - 1 ; \
+         long ci  = (long) cidxA( ci_scalar ? 0 : idx ) - 1 ; \
          while( cx < ci ) cidxX[++cx]= ii; \
       } else { \
          if (assemble_do_sum) \
-            coefX[ii]+=      ( cf_scalar ? coefA(0) : coefA(idx) ); \
+            coefX[ii]+= coefA( cf_scalar ? 0 : idx ); \
          else \
-            coefX[ii] =      ( cf_scalar ? coefA(0) : coefA(idx) ); \
+            coefX[ii] = coefA( cf_scalar ? 0 : idx ); \
       } \
       prev_val= val;\
    } \
@@ -333,6 +350,32 @@ Revision 1.1  2001/03/30 04:34:23  aadler
    SuperMatrix X= create_SuperMatrix( m, n, ii+1, coefX, ridxX, cidxX ); \
    oct_sparse_free( sidx );
 
+// destructively resize a sparse matrix
+#define SPARSE_RESIZE( TYPX, VECTYP1, VECTYP2)			   \
+  /* Special case the redimensioning to an empty matrix */ \
+  if (dv.numel() == 0) \
+    return new octave_sparse (create_SuperMatrix (0, 0, 0, (TYPX *)0, \
+						  0, 0)); \
+\
+  /* Special case the redimensioning of an empty matrix */ \
+  NCformat * Xstore  = (NCformat * ) X.Store; \
+  if (Xstore->nnz == 0) \
+    { \
+      int * colptr = (int *) oct_sparse_malloc ((dv(1) + 1) * sizeof(int)); \
+      for (int i = 0; i <= dv(1); i++) \
+	colptr [i] = 0; \
+      return new octave_sparse (create_SuperMatrix (dv(0), dv(1), 0, \
+						    (TYPX *)0, 0, colptr)); \
+    } \
+\
+  octave_value_list find_val= find(); \
+  int n= (int) dv(1); \
+  int m= (int) dv(0); \
+  ColumnVector ridxA= find_val(0).column_vector_value(); \
+  ColumnVector cidxA= find_val(1).column_vector_value(); \
+  VECTYP1 ## ColumnVector coefA= find_val(2). VECTYP2 ## column_vector_value(); \
+  int assemble_do_sum = 0; \
+  ASSEMBLE_SPARSE( TYPX, false);
 
 // assemble a sparse matrix from full
 //   called by one arg for sparse
@@ -561,3 +604,37 @@ fixrow_comp( const void *i, const void *j)  \
    } /* for n */ \
    cidxX[Unr]= cx; \
    maybe_shrink( cx, nnz, ridxX, coefX );
+
+#ifdef HAVE_OLD_OCTAVE_CONCAT
+#define DEFCATOP_SPARSE_FN(name, ret, t1, t2, e1, e2, f)	\
+  CATOPDECL (name, a1, a2)	     \
+  { \
+    DEBUGMSG("CATOP ## name ## t1 ## t2 ## f"); \
+    CAST_BINOP_ARGS (const octave_ ## t1&, const octave_ ## t2&); \
+    SuperMatrix B = f (v1.e1 (), v2.e2 (), ra_idx); \
+    if (error_state) \
+      return new octave_ ## ret (); \
+    else \
+      return new octave_ ## ret (B); \
+  }
+
+#define INSTALL_SPARSE_CATOP(t1, t2, f) INSTALL_CATOP(t1, t2, f) 
+#elif defined (HAVE_OCTAVE_CONCAT)
+#define DEFCATOP_SPARSE_FN(name, ret, t1, t2, e1, e2, f)	\
+  CATOPDECL (name, a1, a2)	     \
+  { \
+    DEBUGMSG("CATOP ## name ## t1 ## t2 ## f"); \
+    CAST_BINOP_ARGS (octave_ ## t1&, const octave_ ## t2&); \
+    SuperMatrix B = f (v1.e1 (), v2.e2 (), ra_idx); \
+    if (error_state) \
+      return new octave_ ## ret (); \
+    else \
+      return new octave_ ## ret (B); \
+  }
+
+#define INSTALL_SPARSE_CATOP(t1, t2, f) INSTALL_CATOP(t1, t2, f) 
+#else
+#define DEFCATOP_SPARSE_FN(name, ret, t1, t2, e1, e2, f) 
+#define INSTALL_SPARSE_CATOP(t1, t2, f)
+#endif
+
