@@ -16,12 +16,20 @@
 
 ## a = mad(X)
 ##    mean absolute deviation of X
-function a = mad(X)
-  if nargin != 1
-    usage("a = mad (X)");
+function a = mad(X,dim)
+  if nargin == 1
+    dim = min(find(size(X)>1));
+    if isempty(dim), dim=1; endif;
+  endif
+  if (nargin != 1 || nargin != 2)
+    usage("a = mad (X,dim)");
+  elseif (prod(size(X)) != size(X,dim))
+    sz = ones(1,length(size(X));
+    sz(dim) = size(X,dim);
+    a = nanmean (abs (X - repmat (nanmean (X, dim), sz)), dim);
   elseif all (size (X) > 1)
-    a = nanmean (abs (X - ones(size(X,1),1) * nanmean(X)));
+    a = nanmean (abs (X - ones(size(X,1),1) * nanmean(X, dim)), dim);
   else
-    a = nanmean (abs (X - nanmean(X)));
+    a = nanmean (abs (X - nanmean(X, dim)), dim);
   endif
 endfunction
